@@ -30,7 +30,6 @@ import os
 import stat
 import time
 import resource
-import locale
 from functools import partial
 
 
@@ -116,7 +115,7 @@ class solution_parser:
         assert len(sampling_vars) > 0
         a = XorToCNF()
         tmpfname = unique_file("tmp_for_xor_to_cnf_convert")
-        a.convert(fname, tmpfname)
+        a.convert(fname, tmpfname, len(sampling_vars))
 
         with open(tmpfname, "a") as f:
             # NOTE: the "p cnf..." header will be wrong
@@ -134,8 +133,8 @@ class solution_parser:
         print("-> added partial solution to temporary CNF file %s" % tmpfname)
 
         # execute with the other solver
-        toexec = "../../build/tests/minisat/minisat -verb=0 %s" % tmpfname
-        print("Solving with other solver: %s" % toexec)
+        toexec = "../../build/utils/lingeling-ala/lingeling %s" % tmpfname
+        print("sampling check -- solving with other solver: %s" % toexec)
         curr_time = time.time()
         try:
             p = subprocess.Popen(toexec.rsplit(),
@@ -143,7 +142,7 @@ class solution_parser:
                                  preexec_fn=partial(setlimits, self.options.maxtime),
                                  universal_newlines=True)
         except OSError:
-            print("ERROR: Minisat didn't run... weird, it's included as a submodule")
+            print("ERROR: Lingeing didn't run... did you install it? It should be in the path as `lingeling`")
             raise
 
         consoleOutput2 = p.communicate()[0]
@@ -177,8 +176,8 @@ class solution_parser:
         a.convert(fname, tmpfname)
 
         # execute with the other solver
-        toexec = "../../build/tests/minisat/minisat -verb=0 %s" % tmpfname
-        print("Solving with other solver: %s" % toexec)
+        toexec = "../../build/utils/lingeling-ala/lingeling %s" % tmpfname
+        print("UNSAT check -- solving with other solver: %s" % toexec)
         curr_time = time.time()
         try:
             p = subprocess.Popen(toexec.rsplit(),
@@ -186,7 +185,7 @@ class solution_parser:
                                  preexec_fn=partial(setlimits, self.options.maxtime),
                                  universal_newlines=True)
         except OSError:
-            print("ERROR: Minisat didn't run... weird, it's included as a submodule")
+            print("ERROR: Lingeling didn't run... did you install it? It should be in the path as `lingeling`")
             raise
 
         consoleOutput2 = p.communicate()[0]
@@ -206,7 +205,7 @@ class solution_parser:
         # check if the other solver agrees with us
         return otherSolverUNSAT
 
-    def check_debug_lib(self, fname):
+    def check_debug_lib(self, fname, must_check_unsat=True):
         largestPart = self._find_largest_debuglib_part(fname)
         for debugLibPart in range(1, largestPart + 1):
             fname_debug = "%s-debugLibPart%d.output" % (fname, debugLibPart)
@@ -230,21 +229,22 @@ class solution_parser:
                 self.test_found_solution(solution, fname, debugLibPart)
             else:
                 print("debugLib is UNSAT")
-                assert conflict is not None, "debugLibPart must create a conflict in case of UNSAT"
-                self._check_assumps_inside_conflict(assumps, conflict)
-                tmpfname = unique_file("tmp_for_extract_libpart")
-                self._extract_lib_part(fname, debugLibPart, assumps, tmpfname)
+                if must_check_unsat:
+                    assert conflict is not None, "debugLibPart must create a conflict in case of UNSAT"
+                    self._check_assumps_inside_conflict(assumps, conflict)
+                    tmpfname = unique_file("tmp_for_extract_libpart")
+                    self._extract_lib_part(fname, debugLibPart, assumps, tmpfname)
 
-                # check with other solver
-                ret = self.check_unsat(tmpfname)
-                if ret is None:
-                    print("Cannot check, other solver took too much time")
-                elif ret is True:
-                    print("UNSAT verified by other solver")
-                else:
-                    print("Grave bug: SAT-> UNSAT : Other solver found solution!!")
-                    exit(-1)
-                os.unlink(tmpfname)
+                    # check with other solver
+                    ret = self.check_unsat(tmpfname)
+                    if ret is None:
+                        print("Cannot check, other solver took too much time")
+                    elif ret is True:
+                        print("UNSAT verified by other solver")
+                    else:
+                        print("Grave bug: SAT-> UNSAT : Other solver found solution!!")
+                        exit(-1)
+                    os.unlink(tmpfname)
 
         self.remove_debuglib_files(fname)
 
@@ -494,17 +494,19 @@ class solution_parser:
                 return True
 
         # print not set vars
-        print("Unset vars:")
         for lit in lits:
             numlit = int(lit)
             if numlit == 0:
                 break
 
             if abs(numlit) not in solution:
-                print("var %d not set" % abs(numlit))
+                print("var %d in XOR clause not set" % abs(numlit))
 
-        print("Every other var set to FALSE")
+        #print("Every other var set to FALSE")
+        #print("Orig clause in DIMACS: ", lits)
+        print("Error: clause '%s' not satisfied." % line.strip())
         raise NameError("Error: clause '%s' not satisfied." % line)
+        return False
 
     @staticmethod
     def _check_xor_clause(line, solution):
@@ -519,6 +521,7 @@ class solution_parser:
                 final ^= solution[abs(numlit)]
                 final ^= numlit < 0
         if final is False:
+            print("Error: xor-clause '%s' not satisfied." % line.strip())
             raise NameError("Error: xor-clause '%s' not satisfied." % line)
 
         return final
