@@ -35,6 +35,7 @@ THE SOFTWARE.
 #include "time_mem.h"
 #include "varupdatehelper.h"
 #include "watchalgos.h"
+#include "sqlstats.h"
 
 using namespace CMSat;
 using std::cout;
@@ -43,17 +44,21 @@ using std::endl;
 //#define DEBUG_ENQUEUE_LEVEL0
 //#define VERBOSE_DEBUG_POLARITIES
 //#define DEBUG_DYNAMIC_RESTART
+//#define ALTERNATE_WATCH
 
 /**
 @brief Sets a sane default config and allocates handler classes
 */
 PropEngine::PropEngine(
-    const SolverConf* _conf, std::atomic<bool>* _must_interrupt_inter
+    const SolverConf* _conf
+    , Solver* _solver
+    , std::atomic<bool>* _must_interrupt_inter
 ) :
         CNF(_conf, _must_interrupt_inter)
         , order_heap_vsids(VarOrderLt(var_act_vsids))
         , order_heap_maple(VarOrderLt(var_act_maple))
         , qhead(0)
+        , solver(_solver)
 {
 }
 
@@ -64,6 +69,14 @@ PropEngine::~PropEngine()
 void PropEngine::new_var(const bool bva, uint32_t orig_outer)
 {
     CNF::new_var(bva, orig_outer);
+
+    var_act_vsids.insert(var_act_vsids.end(), 1, ActAndOffset());
+    var_act_maple.insert(var_act_maple.end(), 1, ActAndOffset());
+    #ifdef VMTF_NEEDED
+    vmtf_btab.insert(vmtf_btab.end(), 1, 0);
+    vmtf_links.insert(vmtf_links.end(), 1, Link());
+    #endif
+
     //TODO
     //trail... update x->whatever
 }
@@ -71,6 +84,14 @@ void PropEngine::new_var(const bool bva, uint32_t orig_outer)
 void PropEngine::new_vars(size_t n)
 {
     CNF::new_vars(n);
+
+    var_act_vsids.insert(var_act_vsids.end(), n, ActAndOffset());
+    var_act_maple.insert(var_act_maple.end(), n, ActAndOffset());
+    #ifdef VMTF_NEEDED
+    vmtf_btab.insert(vmtf_btab.end(), n, 0);
+    vmtf_links.insert(vmtf_links.end(), n, Link());
+    #endif
+
     //TODO
     //trail... update x->whatever
 }
@@ -78,6 +99,13 @@ void PropEngine::new_vars(size_t n)
 void PropEngine::save_on_var_memory()
 {
     CNF::save_on_var_memory();
+
+    var_act_vsids.resize(nVars());
+    var_act_maple.resize(nVars());
+
+    var_act_vsids.shrink_to_fit();
+    var_act_maple.shrink_to_fit();
+
 }
 
 /**
@@ -202,8 +230,11 @@ bool PropEngine::prop_long_cl_any_order(
         handle_normal_prop_fail(c, offset, confl);
         return false;
     } else {
-        #ifdef STATS_NEEDED
+        #if defined(STATS_NEEDED) || defined(FINAL_PREDICTOR)
         c.stats.propagations_made++;
+        #endif
+        #ifdef STATS_NEEDED
+        c.stats.sum_propagations_made++;
         if (c.red())
             propStats.propsLongRed++;
         else
@@ -266,6 +297,12 @@ PropBy PropEngine::propagate_any_order_fast()
                 *j++ = *i;
                 const lbool val = value(i->lit2());
                 if (val == l_Undef) {
+                    #ifdef STATS_NEEDED
+                    if (i->red())
+                        propStats.propsBinRed++;
+                    else
+                        propStats.propsBinIrred++;
+                    #endif
                     enqueue<false>(i->lit2(), currLevel, PropBy(~p, i->red()));
                     i++;
                 } else if (val == l_False) {
@@ -316,8 +353,20 @@ PropBy PropEngine::propagate_any_order_fast()
             for (uint32_t k = 2; k < c.size(); k++) {
                 //Literal is either unset or satisfied, attach to other watchlist
                 if (likely(value(c[k]) != l_False)) {
+#ifndef ALTERNATE_WATCH
                     c[1] = c[k];
                     c[k] = false_lit;
+#else
+                    //code by Jo Devriendt
+                    if (k >= 4) {
+                        std::swap(c[1], c[k]);
+                        std::swap(c[k/2+1], c[k]);
+                    } else {
+                        c[1] = c[k];
+                        c[k] = false_lit;
+                    }
+#endif
+
                     watches[c[1]].push(w);
                     goto nextClause;
                 }
@@ -342,6 +391,15 @@ PropBy PropEngine::propagate_any_order_fast()
                 assert(j <= end);
                 qhead = trail.size();
             } else {
+                #if defined(FINAL_PREDICTOR) || defined(STATS_NEEDED)
+                c.stats.propagations_made++;
+                #endif
+                #ifdef STATS_NEEDED
+                if (c.red())
+                    propStats.propsLongRed++;
+                else
+                    propStats.propsLongIrred++;
+                #endif
                 if (currLevel == declevel) {
                     enqueue<false>(c[0], currLevel, PropBy(offset));
                 } else {
@@ -456,56 +514,11 @@ void PropEngine::printWatchList(const Lit lit) const
 void PropEngine::updateVars(
     const vector<uint32_t>& outerToInter
     , const vector<uint32_t>& interToOuter
-    , const vector<uint32_t>& interToOuter2
 ) {
-    updateArray(varData, interToOuter);
-    updateArray(assigns, interToOuter);
-    assert(decisionLevel() == 0);
-
     //Trail is NOT correct, only its length is correct
     for(Trail& t: trail) {
         t.lit = lit_Undef;
     }
-    updateBySwap(watches, seen, interToOuter2);
-
-    for(watch_subarray w: watches) {
-        if (!w.empty())
-            updateWatch(w, outerToInter);
-    }
-}
-
-inline void PropEngine::updateWatch(
-    watch_subarray ws
-    , const vector<uint32_t>& outerToInter
-) {
-    for(Watched *it = ws.begin(), *end = ws.end()
-        ; it != end
-        ; ++it
-    ) {
-        if (it->isBin()) {
-            it->setLit2(
-                getUpdatedLit(it->lit2(), outerToInter)
-            );
-            continue;
-        }
-
-        assert(it->isClause());
-        const Clause &cl = *cl_alloc.ptr(it->get_offset());
-        Lit blocked_lit = it->getBlockedLit();
-        blocked_lit = getUpdatedLit(it->getBlockedLit(), outerToInter);
-        bool found = false;
-        for(Lit lit: cl) {
-            if (lit == blocked_lit) {
-                found = true;
-                break;
-            }
-        }
-        if (!found) {
-            it->setBlockedLit(cl[2]);
-        } else {
-            it->setBlockedLit(blocked_lit);
-        }
-    }
 }
 
 void PropEngine::print_trail()
@@ -630,3 +643,99 @@ void PropEngine::load_state(SimpleInFile& f)
 
     CNF::load_state(f);
 }
+
+#ifdef STATS_NEEDED_BRANCH
+void PropEngine::sql_dump_vardata_picktime(uint32_t v, PropBy from)
+{
+    if (!solver->sqlStats)
+        return;
+
+    bool dump = false;
+    double rnd_num = solver->mtrand.randDblExc();
+    if (rnd_num <= conf.dump_individual_cldata_ratio*0.1) {
+        dump = true;
+    }
+    varData[v].dump = dump;
+    if (!dump)
+        return;
+
+    solver->dump_restart_sql(rst_dat_type::var);
+
+    uint64_t outer_var = map_inter_to_outer(v);
+
+    varData[v].sumDecisions_at_picktime = sumDecisions;
+    varData[v].sumConflicts_at_picktime = sumConflicts;
+    varData[v].sumAntecedents_at_picktime = sumAntecedents;
+    varData[v].sumAntecedentsLits_at_picktime = sumAntecedentsLits;
+    varData[v].sumConflictClauseLits_at_picktime = sumConflictClauseLits;
+    varData[v].sumPropagations_at_picktime = sumPropagations;
+    varData[v].sumDecisionBasedCl_at_picktime = sumDecisionBasedCl;
+    varData[v].sumClLBD_at_picktime = sumClLBD;
+    varData[v].sumClSize_at_picktime = sumClSize;
+    double rel_activity_at_picktime =
+        std::log2(var_act_vsids[v]+10e-300)/std::log2(max_vsids_act+10e-300);
+
+    varData[v].last_time_set_was_dec = (from == PropBy());
+
+    //inside data
+    varData[v].inside_conflict_clause_glue_at_picktime = varData[v].inside_conflict_clause_glue;
+    varData[v].inside_conflict_clause_at_picktime = varData[v].inside_conflict_clause;
+    varData[v].inside_conflict_clause_antecedents_at_picktime = varData[v].inside_conflict_clause_antecedents;
+
+    solver->sqlStats->var_data_picktime(
+        solver
+        , outer_var
+        , varData[v]
+        , rel_activity_at_picktime
+    );
+}
+#endif
+
+#ifdef VMTF_NEEDED
+// Update queue to point to last potentially still unassigned variable.
+// All variables after 'queue.unassigned' in bump order are assumed to be
+// assigned.  Then update the 'queue.vmtf_bumped' field and log it.  This is
+// inlined here since it occurs in several inner loops.
+//
+void PropEngine::vmtf_update_queue_unassigned (uint32_t idx) {
+    assert(idx != std::numeric_limits<uint32_t>::max());
+    vmtf_queue.unassigned = idx;
+    vmtf_queue.vmtf_bumped = vmtf_btab[idx];
+}
+
+void PropEngine::vmtf_init_enqueue (uint32_t var) {
+    Link & l = vmtf_links[var];
+    l.next = std::numeric_limits<uint32_t>::max();
+    if (vmtf_queue.last != std::numeric_limits<uint32_t>::max()) {
+        assert(vmtf_links[vmtf_queue.last].next == std::numeric_limits<uint32_t>::max());
+        vmtf_links[vmtf_queue.last].next = var;
+    } else {
+        assert(vmtf_queue.first == std::numeric_limits<uint32_t>::max());
+        vmtf_queue.first = var;
+    }
+    vmtf_btab[var] = ++vmtf_queue.vmtf_bumped;
+    l.prev = vmtf_queue.last;
+    vmtf_queue.last = var;
+    vmtf_update_queue_unassigned(vmtf_queue.last);
+}
+
+// Move vmtf_bumped variables to the front of the (VMTF) decision queue.  The
+// 'vmtf_bumped' time stamp is updated accordingly.  It is used to determine
+// whether the 'queue.assigned' pointer has to be moved in 'unassign'.
+
+void PropEngine::vmtf_bump_queue (uint32_t var) {
+    if (vmtf_links[var].next == std::numeric_limits<uint32_t>::max()) {
+        return;
+    }
+    //Remove from wherever it is, put to the top
+    vmtf_queue.dequeue (vmtf_links, var);
+    vmtf_queue.enqueue (vmtf_links, var);
+
+    assert (vmtf_queue.vmtf_bumped != std::numeric_limits<uint32_t>::max());
+    vmtf_btab[var] = ++vmtf_queue.vmtf_bumped;
+    //LOG ("moved to front variable %d and vmtf_bumped to %" PRId64 "", idx, vmtf_btab[idx]);
+    if (value(var) == l_Undef) {
+        vmtf_update_queue_unassigned(var);
+    }
+}
+#endif
