@@ -64,6 +64,9 @@ uint32_t SubsumeStrengthen::subsume_and_unlink_and_markirred(const ClOffset offs
     if (cl.red()
         && ret.subsumedIrred
     ) {
+        #ifdef STATS_NEEDED
+        solver->stats_del_cl(&cl);
+        #endif
         cl.makeIrred();
         solver->litStats.redLits -= cl.size();
         solver->litStats.irredLits += cl.size();
@@ -153,7 +156,9 @@ SubsumeStrengthen::Sub1Ret SubsumeStrengthen::strengthen_subsume_and_unlink_and_
         ClOffset offset2 = subs[j];
         Clause& cl2 = *solver->cl_alloc.ptr(offset2);
         #ifdef USE_GAUSS
-        if (cl2.used_in_xor()) {
+        if (cl2.used_in_xor() &&
+            solver->conf.force_preserve_xors)
+        {
             continue;
         }
         #endif
@@ -168,6 +173,9 @@ SubsumeStrengthen::Sub1Ret SubsumeStrengthen::strengthen_subsume_and_unlink_and_
             if (cl.red()
                 && !cl2.red()
             ) {
+                #ifdef STATS_NEEDED
+                solver->stats_del_cl(&cl);
+                #endif
                 cl.makeIrred();
                 solver->litStats.redLits -= cl.size();
                 solver->litStats.irredLits += cl.size();
@@ -192,7 +200,9 @@ SubsumeStrengthen::Sub1Ret SubsumeStrengthen::strengthen_subsume_and_unlink_and_
             }
             #endif
             #ifdef USE_GAUSS
-            if (cl2.used_in_xor()) {
+            if (cl2.used_in_xor() &&
+                solver->conf.force_preserve_xors)
+            {
                 continue;
             }
             #endif
@@ -881,7 +891,9 @@ SubsumeStrengthen::Sub1Ret SubsumeStrengthen::backw_sub_str_long_with_implicit(
                 cout << "subsumed clause " << cl2 << endl;
             #endif
             #ifdef USE_GAUSS
-            if (cl2.used_in_xor()) {
+            if (cl2.used_in_xor() &&
+                solver->conf.force_preserve_xors)
+            {
                 continue;
             }
             #endif
@@ -899,7 +911,9 @@ SubsumeStrengthen::Sub1Ret SubsumeStrengthen::backw_sub_str_long_with_implicit(
             }
             #endif
             #ifdef USE_GAUSS
-            if (cl2.used_in_xor()) {
+            if (cl2.used_in_xor() &&
+                solver->conf.force_preserve_xors)
+            {
                 //cout << "str-ing used in XOR with bin" << endl;
                 continue;
             }
